@@ -64,8 +64,10 @@ static dev_t new_decode_dev(uint32_t dev)
 class CephFuse::Handle {
 public:
   Handle(Client *c, int fd);
+  ~Handle();
 
   int init(int argc, const char *argv[]);
+  int start();
   int loop();
   void finalize();
 
@@ -88,6 +90,7 @@ public:
   ceph::unordered_map<uint64_t,int> snap_stag_map;
   ceph::unordered_map<int,uint64_t> stag_snap_map;
 
+  struct fuse_args args;
 };
 
 static void fuse_ll_lookup(fuse_req_t req, fuse_ino_t parent, const char *name)
@@ -636,69 +639,6 @@ static void fuse_ll_statfs(fuse_req_t req, fuse_ino_t ino)
   cfuse->iput(in); // iput required
 }
 
-static void fuse_ll_getlk(fuse_req_t req, fuse_ino_t ino,
-			  struct fuse_file_info *fi, struct flock *lock)
-{
-  CephFuse::Handle *cfuse = (CephFuse::Handle *)fuse_req_userdata(req);
-  Fh *fh = (Fh*)fi->fh;
-
-  int r = cfuse->client->ll_getlk(fh, lock, fi->lock_owner);
-  if (r == 0)
-    fuse_reply_lock(req, lock);
-  else
-    fuse_reply_err(req, -r);
-}
-
-static void fuse_ll_setlk(fuse_req_t req, fuse_ino_t ino,
-		          struct fuse_file_info *fi, struct flock *lock, int sleep)
-{
-  CephFuse::Handle *cfuse = (CephFuse::Handle *)fuse_req_userdata(req);
-  Fh *fh = (Fh*)fi->fh;
-
-  // must use multithread if operation may block
-  if (!cfuse->client->cct->_conf->fuse_multithreaded &&
-      sleep && lock->l_type != F_UNLCK) {
-    fuse_reply_err(req, EDEADLK);
-    return;
-  }
-
-  int r = cfuse->client->ll_setlk(fh, lock, fi->lock_owner, sleep, req);
-  fuse_reply_err(req, -r);
-}
-
-static void fuse_ll_interrupt(fuse_req_t req, void* data)
-{
-  CephFuse::Handle *cfuse = (CephFuse::Handle *)fuse_req_userdata(req);
-  cfuse->client->ll_interrupt(data);
-}
-
-static void switch_interrupt_cb(void *req, void* data)
-{
-  if (data)
-    fuse_req_interrupt_func((fuse_req_t)req, fuse_ll_interrupt, data);
-  else
-    fuse_req_interrupt_func((fuse_req_t)req, NULL, NULL);
-}
-
-#if FUSE_VERSION >= FUSE_MAKE_VERSION(2, 9)
-static void fuse_ll_flock(fuse_req_t req, fuse_ino_t ino,
-		          struct fuse_file_info *fi, int cmd)
-{
-  CephFuse::Handle *cfuse = (CephFuse::Handle *)fuse_req_userdata(req);
-  Fh *fh = (Fh*)fi->fh;
-
-  // must use multithread if operation may block
-  if (!cfuse->client->cct->_conf->fuse_multithreaded &&
-      !(cmd & (LOCK_NB | LOCK_UN))) {
-    fuse_reply_err(req, EDEADLK);
-    return;
-  }
-
-  int r = cfuse->client->ll_flock(fh, cmd, fi->lock_owner, req);
-  fuse_reply_err(req, -r);
-}
-#endif
-
 #if 0
 static int getgroups_cb(void *handle, uid_t uid, gid_t **sgids)
 {
@@ -805,8 +745,8 @@ const static struct fuse_lowlevel_ops fuse_ll_oper = {
  removexattr: fuse_ll_removexattr,
  access: fuse_ll_access,
  create: fuse_ll_create,
- getlk: fuse_ll_getlk,
- setlk: fuse_ll_setlk,
+ getlk: 0,
+ setlk: 0,
  bmap: 0,
 #if FUSE_VERSION >= FUSE_MAKE_VERSION(2, 8)
 #ifdef FUSE_IOCTL_COMPAT
@@ -815,16 +755,14 @@ const static struct fuse_lowlevel_ops fuse_ll_oper = {
  ioctl: 0,
 #endif
  poll: 0,
-#endif
-#if FUSE_VERSION >= FUSE_MAKE_VERSION(2, 9)
+#if FUSE_VERSION > FUSE_MAKE_VERSION(2, 9)
  write_buf: 0,
  retrieve_reply: 0,
  forget_multi: 0,
- flock: fuse_ll_flock,
-#endif
-#if FUSE_VERSION > FUSE_MAKE_VERSION(2, 9)
+ flock: 0,
  fallocate: fuse_ll_fallocate
 #endif
+#endif
 };
 
 
@@ -839,6 +777,12 @@ CephFuse::Handle::Handle(Client *c, int fd) :
 {
   snap_stag_map[CEPH_NOSNAP] = 0;
   stag_snap_map[0] = CEPH_NOSNAP;
+  memset(&args, 0, sizeof(args));
+}
+
+CephFuse::Handle::~Handle()
+{
+  fuse_opt_free_args(&args);
 }
 
 void CephFuse::Handle::finalize()
@@ -890,42 +834,45 @@ int CephFuse::Handle::init(int argc, const char *argv[])
   for (int argctr = 1; argctr < argc; argctr++)
     newargv[newargc++] = argv[argctr];
 
-  struct fuse_args args = FUSE_ARGS_INIT(newargc, (char**)newargv);
-  int ret = 0;
+  derr << "init, newargv = " << newargv << " newargc=" << newargc << dendl;
+  struct fuse_args a = FUSE_ARGS_INIT(newargc, (char**)newargv);
+  args = a;  // Roundabout construction b/c FUSE_ARGS_INIT is for initialization not assignment
 
-  char *mountpoint;
   if (fuse_parse_cmdline(&args, &mountpoint, NULL, NULL) == -1) {
     derr << "fuse_parse_cmdline failed." << dendl;
-    ret = EINVAL;
-    goto done;
+    fuse_opt_free_args(&args);
+    free(newargv);
+    return EINVAL;
   }
 
+  assert(args.allocated);  // Checking fuse has realloc'd args so we can free newargv
+  free(newargv);
+  return 0;
+}
+
+int CephFuse::Handle::start()
+{
   ch = fuse_mount(mountpoint, &args);
   if (!ch) {
     derr << "fuse_mount(mountpoint=" << mountpoint << ") failed." << dendl;
-    ret = EIO;
-    goto done;
+    return EIO;
   }
 
   se = fuse_lowlevel_new(&args, &fuse_ll_oper, sizeof(fuse_ll_oper), this);
   if (!se) {
     derr << "fuse_lowlevel_new failed" << dendl;
-    ret = EDOM;
-    goto done;
+    return EDOM;
   }
 
   signal(SIGTERM, SIG_DFL);
   signal(SIGINT, SIG_DFL);
   if (fuse_set_signal_handlers(se) == -1) {
     derr << "fuse_set_signal_handlers failed" << dendl;
-    ret = ENOSYS;
-    goto done;
+    return ENOSYS;
   }
 
   fuse_session_add_chan(se, ch);
 
-  client->ll_register_switch_interrupt_cb(switch_interrupt_cb);
-
   /*
    * this is broken:
    *
@@ -944,10 +891,7 @@ int CephFuse::Handle::init(int argc, const char *argv[])
   if (client->cct->_conf->fuse_use_invalidate_cb)
     client->ll_register_ino_invalidate_cb(ino_invalidate_cb, this);
 
-done:
-  fuse_opt_free_args(&args);
-  free(newargv);
-  return ret;
+  return 0;
 }
 
 int CephFuse::Handle::loop()
@@ -1018,6 +962,11 @@ int CephFuse::init(int argc, const char *argv[])
   return _handle->init(argc, argv);
 }
 
+int CephFuse::start()
+{
+  return _handle->start();
+}
+
 int CephFuse::loop()
 {
   return _handle->loop();
@@ -1027,3 +976,12 @@ void CephFuse::finalize()
 {
   return _handle->finalize();
 }
+
+std::string CephFuse::get_mount_point() const
+{
+  if (_handle->mountpoint) {
+    return _handle->mountpoint;
+  } else {
+    return "";
+  }
+}
