@@ -58,15 +58,15 @@ public:
   friend class MOSDOpReply;
 
   // read
-  snapid_t get_snapid() { return snapid; }
-  void set_snapid(snapid_t s) { snapid = s; }
+  const snapid_t& get_snapid() { return snapid; }
+  void set_snapid(const snapid_t& s) { snapid = s; }
   // writ
-  snapid_t get_snap_seq() const { return snap_seq; }
+  const snapid_t& get_snap_seq() const { return snap_seq; }
   const vector<snapid_t> &get_snaps() const { return snaps; }
   void set_snaps(const vector<snapid_t>& i) {
     snaps = i;
   }
-  void set_snap_seq(snapid_t s) { snap_seq = s; }
+  void set_snap_seq(const snapid_t& s) { snap_seq = s; }
 
   osd_reqid_t get_reqid() const {
     return osd_reqid_t(get_orig_source(),
@@ -78,22 +78,22 @@ public:
   
   object_t& get_oid() { return oid; }
 
-  pg_t     get_pg() const { return pgid; }
+  const pg_t&     get_pg() const { return pgid; }
 
-  object_locator_t get_object_locator() const {
+  const object_locator_t& get_object_locator() const {
     return oloc;
   }
 
   epoch_t  get_map_epoch() { return osdmap_epoch; }
 
-  eversion_t get_version() { return reassert_version; }
+  const eversion_t& get_version() { return reassert_version; }
   
   utime_t get_mtime() { return mtime; }
 
   MOSDOp()
     : Message(CEPH_MSG_OSD_OP, HEAD_VERSION, COMPAT_VERSION) { }
   MOSDOp(int inc, long tid,
-         object_t& _oid, object_locator_t& _oloc, pg_t _pgid, epoch_t _osdmap_epoch,
+         object_t& _oid, object_locator_t& _oloc, pg_t& _pgid, epoch_t _osdmap_epoch,
 	 int _flags)
     : Message(CEPH_MSG_OSD_OP, HEAD_VERSION, COMPAT_VERSION),
       client_inc(inc),
