@@ -97,6 +97,8 @@ struct RGWRequest
   RGWRequest() : id(0), s(NULL), op(NULL) {
   }
 
+  virtual ~RGWRequest() {}
+
   void init_state(req_state *_s) {
     s = _s;
   }
@@ -206,6 +208,9 @@ protected:
       perfcounter->inc(l_rgw_qactive, -1);
     }
     void _dump_queue() {
+      if (!g_conf->subsys.should_gather(ceph_subsys_rgw, 20)) {
+        return;
+      }
       deque<RGWRequest *>::iterator iter;
       if (process->m_req_queue.empty()) {
         dout(20) << "RGWWQ: empty" << dendl;
@@ -728,11 +733,11 @@ int usage()
   cerr << "  --rgw-region=<region>     region in which radosgw runs\n";
   cerr << "  --rgw-zone=<zone>         zone in which radosgw runs\n";
   cerr << "  --rgw-socket-path=<path>  specify a unix domain socket path\n";
-  generic_server_usage();
   cerr << "  -m monaddress[:port]      connect to specified monitor\n";
   cerr << "  --keyring=<path>          path to radosgw keyring\n";
   cerr << "  --logfile=<logfile>       file to log debug output\n";
   cerr << "  --debug-rgw=<log-level>/<memory-level>  set radosgw debug level\n";
+  generic_server_usage();
 
   return 0;
 }
@@ -978,7 +983,7 @@ int main(int argc, const char **argv)
   vector<const char *> def_args;
   def_args.push_back("--debug-rgw=1/5");
   def_args.push_back("--keyring=$rgw_data/keyring");
-  def_args.push_back("--log-file=/var/log/radosgw/$cluster-$name");
+  def_args.push_back("--log-file=/var/log/radosgw/$cluster-$name.log");
 
   vector<const char*> args;
   argv_to_vec(argc, argv, args);
