@@ -79,7 +79,7 @@ TEST(cls_rbd, copyup)
   size_t l = 4 << 20;
   char *b = random_buf(l);
   inbl.append(b, l);
-  delete b;
+  delete [] b;
   ASSERT_EQ(l, inbl.length());
 
   // copyup to nonexistent object should create new object
@@ -94,7 +94,7 @@ TEST(cls_rbd, copyup)
   bufferlist inbl2;
   b = random_buf(l);
   inbl2.append(b, l);
-  delete b;
+  delete [] b;
   ASSERT_EQ(l, inbl2.length());
 
   // should still succeed
@@ -869,7 +869,7 @@ TEST(cls_rbd, snapid_race)
   bl.append(bp);
 
   string oid = "foo";
-  ASSERT_EQ(4096, ioctx.write(oid, bl, 4096, 0));
+  ASSERT_EQ(0, ioctx.write(oid, bl, 4096, 0));
   ASSERT_EQ(0, old_snapshot_add(&ioctx, oid, 1, "test1"));
   ASSERT_EQ(0, old_snapshot_add(&ioctx, oid, 3, "test3"));
   ASSERT_EQ(-ESTALE, old_snapshot_add(&ioctx, oid, 2, "test2"));
