@@ -2,8 +2,10 @@
  * Ceph - scalable distributed file system
  *
  * Copyright (C) 2014 CERN (Switzerland)
+ * Copyright (C) 2014 Red Hat <contact@redhat.com>
  *
  * Author: Andreas-Joachim Peters <Andreas.Joachim.Peters@cern.ch>
+ * Author: Loic Dachary <loic@dachary.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -18,52 +20,53 @@
 #include "include/stringify.h"
 #include "global/global_init.h"
 #include "erasure-code/isa/ErasureCodeIsa.h"
+#include "erasure-code/isa/xor_op.h"
 #include "common/ceph_argparse.h"
 #include "global/global_context.h"
 #include "gtest/gtest.h"
 
-template <typename T>
+ErasureCodeIsaTableCache tcache;
+
 class IsaErasureCodeTest : public ::testing::Test {
 public:
+  void compare_chunks(bufferlist &in, map<int, bufferlist> &encoded);
+  void encode_decode(unsigned object_size); 
 };
 
-typedef ::testing::Types<
-ErasureCodeIsaDefault
-> IsaTypes;
-TYPED_TEST_CASE(IsaErasureCodeTest, IsaTypes);
-
-TYPED_TEST(IsaErasureCodeTest, encode_decode)
+void IsaErasureCodeTest::compare_chunks(bufferlist &in, map<int, bufferlist> &encoded)
 {
-  TypeParam Isa;
+  unsigned object_size = in.length();
+  unsigned chunk_size = encoded[0].length();
+  for (unsigned i = 0; i < encoded.size(); i++) {
+    if (i * chunk_size >= object_size)
+      break;
+    int chunk_length = object_size > (i + 1) * chunk_size ? chunk_size : object_size - i * chunk_size;
+    EXPECT_EQ(0, memcmp(encoded[i].c_str(), in.c_str() + i * chunk_size, chunk_length));
+  }
+}
+
+void IsaErasureCodeTest::encode_decode(unsigned object_size)
+{
+  ErasureCodeIsaDefault Isa(tcache);
+
   map<std::string, std::string> parameters;
   parameters["k"] = "2";
   parameters["m"] = "2";
   Isa.init(parameters);
 
-#define LARGE_ENOUGH 2048
-  bufferptr in_ptr(buffer::create_page_aligned(LARGE_ENOUGH));
-  in_ptr.zero();
-  in_ptr.set_length(0);
-  const char *payload =
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
-  in_ptr.append(payload, strlen(payload));
+  string payload(object_size, 'X');
   bufferlist in;
-  in.push_front(in_ptr);
+  // may be multiple bufferptr if object_size is larger than CEPH_PAGE_SIZE
+  in.append(payload.c_str(), payload.length());
   int want_to_encode[] = {0, 1, 2, 3};
   map<int, bufferlist> encoded;
   EXPECT_EQ(0, Isa.encode(set<int>(want_to_encode, want_to_encode + 4),
                           in,
                           &encoded));
   EXPECT_EQ(4u, encoded.size());
-  unsigned length = encoded[0].length();
-  EXPECT_EQ(0, strncmp(encoded[0].c_str(), in.c_str(), length));
-  EXPECT_EQ(0, strncmp(encoded[1].c_str(), in.c_str() + length,
-                       in.length() - length));
-
+  unsigned chunk_size = encoded[0].length();
+  EXPECT_EQ(chunk_size, Isa.get_chunk_size(object_size));
+  compare_chunks(in, encoded);
 
   // all chunks are available
   {
@@ -73,20 +76,15 @@ TYPED_TEST(IsaErasureCodeTest, encode_decode)
                             encoded,
                             &decoded));
     EXPECT_EQ(2u, decoded.size());
-    EXPECT_EQ(length, decoded[0].length());
-    EXPECT_EQ(0, strncmp(decoded[0].c_str(), in.c_str(), length));
-    EXPECT_EQ(0, strncmp(decoded[1].c_str(), in.c_str() + length,
-                         in.length() - length));
+    EXPECT_EQ(chunk_size, decoded[0].length());
+    compare_chunks(in, decoded);
   }
 
   // one data chunk is missing
   {
     map<int, bufferlist> degraded = encoded;
 
-    buffer::ptr enc1(buffer::create_page_aligned(LARGE_ENOUGH));
-    enc1.zero();
-    enc1.set_length(0);
-    enc1.append(encoded[1].c_str(), length);
+    string enc1(encoded[1].c_str(), chunk_size);
 
     degraded.erase(1);
     EXPECT_EQ(3u, degraded.size());
@@ -97,18 +95,15 @@ TYPED_TEST(IsaErasureCodeTest, encode_decode)
                             &decoded));
     // always decode all, regardless of want_to_decode
     EXPECT_EQ(4u, decoded.size());
-    EXPECT_EQ(length, decoded[1].length());
-    EXPECT_EQ(0, strncmp(decoded[1].c_str(), enc1.c_str(), length));
+    EXPECT_EQ(chunk_size, decoded[1].length());
+    EXPECT_EQ(0, memcmp(decoded[1].c_str(), enc1.c_str(), chunk_size));
   }
 
   // non-xor coding chunk is missing
   {
     map<int, bufferlist> degraded = encoded;
 
-    buffer::ptr enc3(buffer::create_page_aligned(LARGE_ENOUGH));
-    enc3.zero();
-    enc3.set_length(0);
-    enc3.append(encoded[3].c_str(), length);
+    string enc3(encoded[3].c_str(), chunk_size);
 
     degraded.erase(3);
     EXPECT_EQ(3u, degraded.size());
@@ -119,18 +114,15 @@ TYPED_TEST(IsaErasureCodeTest, encode_decode)
                             &decoded));
     // always decode all, regardless of want_to_decode
     EXPECT_EQ(4u, decoded.size());
-    EXPECT_EQ(length, decoded[3].length());
-    EXPECT_EQ(0, strncmp(decoded[3].c_str(), enc3.c_str(), length));
+    EXPECT_EQ(chunk_size, decoded[3].length());
+    EXPECT_EQ(0, memcmp(decoded[3].c_str(), enc3.c_str(), chunk_size));
   }
 
   // xor coding chunk is missing
   {
     map<int, bufferlist> degraded = encoded;
 
-    buffer::ptr enc2(buffer::create_page_aligned(LARGE_ENOUGH));
-    enc2.zero();
-    enc2.set_length(0);
-    enc2.append(encoded[2].c_str(), length);
+    string enc2(encoded[2].c_str(), chunk_size);
 
     degraded.erase(2);
     EXPECT_EQ(3u, degraded.size());
@@ -141,18 +133,15 @@ TYPED_TEST(IsaErasureCodeTest, encode_decode)
                             &decoded));
     // always decode all, regardless of want_to_decode
     EXPECT_EQ(4u, decoded.size());
-    EXPECT_EQ(length, decoded[2].length());
-    EXPECT_EQ(0, strncmp(decoded[2].c_str(), enc2.c_str(), length));
+    EXPECT_EQ(chunk_size, decoded[2].length());
+    EXPECT_EQ(0, memcmp(decoded[2].c_str(), enc2.c_str(), chunk_size));
   }
 
   // one data and one coding chunk is missing
   {
     map<int, bufferlist> degraded = encoded;
 
-    buffer::ptr enc3(buffer::create_page_aligned(LARGE_ENOUGH));
-    enc3.zero();
-    enc3.set_length(0);
-    enc3.append(encoded[3].c_str(), length);
+    string enc3(encoded[3].c_str(), chunk_size);
 
     degraded.erase(1);
     degraded.erase(3);
@@ -164,8 +153,8 @@ TYPED_TEST(IsaErasureCodeTest, encode_decode)
                             &decoded));
     // always decode all, regardless of want_to_decode
     EXPECT_EQ(4u, decoded.size());
-    EXPECT_EQ(length, decoded[1].length());
-    EXPECT_EQ(0, strncmp(decoded[3].c_str(), enc3.c_str(), length));
+    EXPECT_EQ(chunk_size, decoded[1].length());
+    EXPECT_EQ(0, memcmp(decoded[3].c_str(), enc3.c_str(), chunk_size));
   }
 
   // two data chunks are missing
@@ -181,16 +170,25 @@ TYPED_TEST(IsaErasureCodeTest, encode_decode)
                             &decoded));
     // always decode all, regardless of want_to_decode
     EXPECT_EQ(4u, decoded.size());
-    EXPECT_EQ(length, decoded[0].length());
-    EXPECT_EQ(0, strncmp(decoded[0].c_str(), in.c_str(), length));
-    EXPECT_EQ(0, strncmp(decoded[1].c_str(), in.c_str() + length,
-                         in.length() - length));
+    EXPECT_EQ(chunk_size, decoded[0].length());
+    compare_chunks(in, decoded);
   }
+
 }
 
-TYPED_TEST(IsaErasureCodeTest, minimum_to_decode)
+TEST_F(IsaErasureCodeTest, encode_decode)
 {
-  TypeParam Isa;
+  encode_decode(1);
+  encode_decode(EC_ISA_ADDRESS_ALIGNMENT);
+  encode_decode(EC_ISA_ADDRESS_ALIGNMENT + 1);
+  encode_decode(2048);
+  encode_decode(4096);
+  encode_decode(4096 + 1);
+}
+
+TEST_F(IsaErasureCodeTest, minimum_to_decode)
+{
+  ErasureCodeIsaDefault Isa(tcache);
   map<std::string, std::string> parameters;
   parameters["k"] = "2";
   parameters["m"] = "2";
@@ -284,54 +282,86 @@ TYPED_TEST(IsaErasureCodeTest, minimum_to_decode)
   }
 }
 
-TEST(IsaErasureCodeTest, encode)
+TEST_F(IsaErasureCodeTest, chunk_size)
 {
-  ErasureCodeIsaDefault Isa;
+  {
+    ErasureCodeIsaDefault Isa(tcache);
+    map<std::string, std::string> parameters;
+    parameters["k"] = "2";
+    parameters["m"] = "1";
+    Isa.init(parameters);
+    int k = 2;
+
+    ASSERT_EQ(EC_ISA_ADDRESS_ALIGNMENT, Isa.get_chunk_size(1));
+    ASSERT_EQ(EC_ISA_ADDRESS_ALIGNMENT, Isa.get_chunk_size(EC_ISA_ADDRESS_ALIGNMENT * k - 1));
+    ASSERT_EQ(EC_ISA_ADDRESS_ALIGNMENT * 2, Isa.get_chunk_size(EC_ISA_ADDRESS_ALIGNMENT * k + 1));
+  }
+  {
+    ErasureCodeIsaDefault Isa(tcache);
+    map<std::string, std::string> parameters;
+    parameters["k"] = "3";
+    parameters["m"] = "1";
+    Isa.init(parameters);
+    int k = 3;
+
+    ASSERT_EQ(EC_ISA_ADDRESS_ALIGNMENT, Isa.get_chunk_size(1));
+    ASSERT_EQ(EC_ISA_ADDRESS_ALIGNMENT, Isa.get_chunk_size(EC_ISA_ADDRESS_ALIGNMENT * k - 1));
+    ASSERT_EQ(EC_ISA_ADDRESS_ALIGNMENT * 2, Isa.get_chunk_size(EC_ISA_ADDRESS_ALIGNMENT * k + 1));
+    int object_size = EC_ISA_ADDRESS_ALIGNMENT * k * 1024 + 1;
+    ASSERT_NE(0, object_size % k);
+    ASSERT_NE(0, object_size % EC_ISA_ADDRESS_ALIGNMENT);
+    int chunk_size = Isa.get_chunk_size(object_size);
+    ASSERT_EQ(0, chunk_size % EC_ISA_ADDRESS_ALIGNMENT);
+    ASSERT_GT(chunk_size, (chunk_size * k) - object_size);
+  }
+}
+
+TEST_F(IsaErasureCodeTest, encode)
+{
+  ErasureCodeIsaDefault Isa(tcache);
   map<std::string, std::string> parameters;
   parameters["k"] = "2";
   parameters["m"] = "2";
   Isa.init(parameters);
 
-  unsigned alignment = Isa.get_alignment();
+  unsigned aligned_object_size = Isa.get_alignment() * 2;
   {
     //
     // When the input bufferlist needs to be padded because
     // it is not properly aligned, it is padded with zeros.
     //
     bufferlist in;
-    map<int, bufferlist> encoded;
-    int want_to_encode[] = {0, 1, 2, 3};
-    int trail_length = 2;
-    in.append(string(alignment + trail_length, 'X'));
-    EXPECT_EQ(0, Isa.encode(set<int>(want_to_encode, want_to_encode + 4),
-                            in,
-                            &encoded));
+    map<int,bufferlist> encoded;
+    int want_to_encode[] = { 0, 1, 2, 3 };
+    int trail_length = 1;
+    in.append(string(aligned_object_size + trail_length, 'X'));
+    EXPECT_EQ(0, Isa.encode(set<int>(want_to_encode, want_to_encode+4),
+				 in,
+				 &encoded));
     EXPECT_EQ(4u, encoded.size());
-    for (int i = 0; i < 4; i++)
-      EXPECT_EQ(alignment, encoded[i].length());
     char *last_chunk = encoded[1].c_str();
+    int length =encoded[1].length();
     EXPECT_EQ('X', last_chunk[0]);
-    EXPECT_EQ('\0', last_chunk[trail_length]);
+    EXPECT_EQ('\0', last_chunk[length - trail_length]);
   }
 
   {
     //
     // When only the first chunk is required, the encoded map only
-    // contains the first chunk. Although the encode
+    // contains the first chunk. Although the Isa encode
     // internally allocated a buffer because of padding requirements
     // and also computes the coding chunks, they are released before
     // the return of the method, as shown when running the tests thru
     // valgrind (there is no leak).
     //
     bufferlist in;
-    map<int, bufferlist> encoded;
+    map<int,bufferlist> encoded;
     set<int> want_to_encode;
     want_to_encode.insert(0);
-    int trail_length = 2;
-    in.append(string(alignment + trail_length, 'X'));
+    int trail_length = 1;
+    in.append(string(aligned_object_size + trail_length, 'X'));
     EXPECT_EQ(0, Isa.encode(want_to_encode, in, &encoded));
     EXPECT_EQ(1u, encoded.size());
-    EXPECT_EQ(alignment, encoded[0].length());
   }
 }
 
@@ -348,18 +378,18 @@ DecodeAndVerify(ErasureCodeIsaDefault& Isa, map<int, bufferlist> &degraded, set<
 
   for (int i = 0; i < (int) decoded.size(); i++) {
     // compare all the buffers with their original
-    ok |= strncmp(decoded[i].c_str(), enc[i].c_str(), length);
+    ok |= memcmp(decoded[i].c_str(), enc[i].c_str(), length);
   }
 
   return ok;
 }
 
-TYPED_TEST(IsaErasureCodeTest, isa_vandermonde_exhaustive)
+TEST_F(IsaErasureCodeTest, isa_vandermonde_exhaustive)
 {
   // Test all possible failure scenarios and reconstruction cases for
   // a (12,4) configuration using the vandermonde matrix
 
-  TypeParam Isa;
+  ErasureCodeIsaDefault Isa(tcache);
   map<std::string, std::string> parameters;
   parameters["k"] = "12";
   parameters["m"] = "4";
@@ -419,7 +449,7 @@ TYPED_TEST(IsaErasureCodeTest, isa_vandermonde_exhaustive)
   unsigned length = encoded[0].length();
 
   for (int i = 0; i < k; i++) {
-    EXPECT_EQ(0, strncmp(encoded[i].c_str(), in.c_str() + (i * length), length));
+    EXPECT_EQ(0, memcmp(encoded[i].c_str(), in.c_str() + (i * length), length));
   }
 
   buffer::ptr enc[k + m];
@@ -477,14 +507,14 @@ TYPED_TEST(IsaErasureCodeTest, isa_vandermonde_exhaustive)
     want_to_decode.erase(l1);
   }
   EXPECT_EQ(2516, cnt_cf);
-  EXPECT_EQ(2503, Isa.get_tbls_lru_size());
+  EXPECT_EQ(2506, tcache.getDecodingTableCacheSize()); // 3 entries from (2,2) test and 2503 from (12,4)
 }
 
-TYPED_TEST(IsaErasureCodeTest, isa_cauchy_exhaustive)
+TEST_F(IsaErasureCodeTest, isa_cauchy_exhaustive)
 {
   // Test all possible failure scenarios and reconstruction cases for
   // a (12,4) configuration using the cauchy matrix
-  TypeParam Isa(ErasureCodeIsaDefault::kCauchy);
+  ErasureCodeIsaDefault Isa(tcache,ErasureCodeIsaDefault::kCauchy);
   map<std::string, std::string> parameters;
   parameters["k"] = "12";
   parameters["m"] = "4";
@@ -546,7 +576,7 @@ TYPED_TEST(IsaErasureCodeTest, isa_cauchy_exhaustive)
   unsigned length = encoded[0].length();
 
   for (int i = 0; i < k; i++) {
-    EXPECT_EQ(0, strncmp(encoded[i].c_str(), in.c_str() + (i * length), length));
+    EXPECT_EQ(0, memcmp(encoded[i].c_str(), in.c_str() + (i * length), length));
   }
 
   buffer::ptr enc[k + m];
@@ -604,15 +634,142 @@ TYPED_TEST(IsaErasureCodeTest, isa_cauchy_exhaustive)
     want_to_decode.erase(l1);
   }
   EXPECT_EQ(2516, cnt_cf);
-  EXPECT_EQ(2516, Isa.get_tbls_lru_size());
+  EXPECT_EQ(2516, tcache.getDecodingTableCacheSize(ErasureCodeIsaDefault::kCauchy));
 }
 
-TYPED_TEST(IsaErasureCodeTest, isa_xor_codec)
+TEST_F(IsaErasureCodeTest, isa_cauchy_cache_trash)
+{
+  // Test all possible failure scenarios and reconstruction cases for
+  // a (12,4) configuration using the cauchy matrix
+  ErasureCodeIsaDefault Isa(tcache,ErasureCodeIsaDefault::kCauchy);
+  map<std::string, std::string> parameters;
+  parameters["k"] = "16";
+  parameters["m"] = "4";
+  parameters["technique"] = "cauchy";
+
+  Isa.init(parameters);
+
+  int k = 16;
+  int m = 4;
+
+#define LARGE_ENOUGH 2048
+  bufferptr in_ptr(buffer::create_page_aligned(LARGE_ENOUGH));
+  in_ptr.zero();
+  in_ptr.set_length(0);
+  const char *payload =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
+  in_ptr.append(payload, strlen(payload));
+  bufferlist in;
+  in.push_front(in_ptr);
+
+  set<int>want_to_encode;
+
+  map<int, bufferlist> encoded;
+  for (int i = 0; i < (k + m); i++) {
+    want_to_encode.insert(i);
+  }
+
+
+  EXPECT_EQ(0, Isa.encode(want_to_encode,
+                          in,
+                          &encoded));
+
+  EXPECT_EQ((unsigned) (k + m), encoded.size());
+
+  unsigned length = encoded[0].length();
+
+  for (int i = 0; i < k; i++) {
+    EXPECT_EQ(0, memcmp(encoded[i].c_str(), in.c_str() + (i * length), length));
+  }
+
+  buffer::ptr enc[k + m];
+  // create buffers with a copy of the original data to be able to compare it after decoding
+  {
+    for (int i = 0; i < (k + m); i++) {
+      buffer::ptr newenc(buffer::create_page_aligned(LARGE_ENOUGH));
+      enc[i] = newenc;
+      enc[i].zero();
+      enc[i].set_length(0);
+      enc[i].append(encoded[i].c_str(), length);
+    }
+  }
+
+  // loop through all possible loss scenarios
+  bool err = true;
+  int cnt_cf = 0;
+
+  for (int l1 = 0; l1 < (k + m); l1++) {
+    map<int, bufferlist> degraded = encoded;
+    set<int> want_to_decode;
+    degraded.erase(l1);
+    want_to_decode.insert(l1);
+    err = DecodeAndVerify(Isa, degraded, want_to_decode, enc, length);
+    EXPECT_EQ(0, err);
+    cnt_cf++;
+    for (int l2 = l1 + 1; l2 < (k + m); l2++) {
+      degraded.erase(l2);
+      want_to_decode.insert(l2);
+      err = DecodeAndVerify(Isa, degraded, want_to_decode, enc, length);
+      EXPECT_EQ(0, err);
+      cnt_cf++;
+      for (int l3 = l2 + 1; l3 < (k + m); l3++) {
+        degraded.erase(l3);
+        want_to_decode.insert(l3);
+        err = DecodeAndVerify(Isa, degraded, want_to_decode, enc, length);
+        EXPECT_EQ(0, err);
+        cnt_cf++;
+        for (int l4 = l3 + 1; l4 < (k + m); l4++) {
+          degraded.erase(l4);
+          want_to_decode.insert(l4);
+          err = DecodeAndVerify(Isa, degraded, want_to_decode, enc, length);
+          EXPECT_EQ(0, err);
+          degraded[l4] = encoded[l4];
+          want_to_decode.erase(l4);
+          cnt_cf++;
+        }
+        degraded[l3] = encoded[l3];
+        want_to_decode.erase(l3);
+      }
+      degraded[l2] = encoded[l2];
+      want_to_decode.erase(l2);
+    }
+    degraded[l1] = encoded[l1];
+    want_to_decode.erase(l1);
+  }
+  EXPECT_EQ(6195, cnt_cf);
+  EXPECT_EQ(2516, tcache.getDecodingTableCacheSize(ErasureCodeIsaDefault::kCauchy));
+}
+
+TEST_F(IsaErasureCodeTest, isa_xor_codec)
 {
   // Test all possible failure scenarios and reconstruction cases for
   // a (4,1) RAID-5 like configuration 
 
-  TypeParam Isa;
+  ErasureCodeIsaDefault Isa(tcache);
   map<std::string, std::string> parameters;
   parameters["k"] = "4";
   parameters["m"] = "1";
@@ -672,7 +829,7 @@ TYPED_TEST(IsaErasureCodeTest, isa_xor_codec)
   unsigned length = encoded[0].length();
 
   for (int i = 0; i < k; i++) {
-    EXPECT_EQ(0, strncmp(encoded[i].c_str(), in.c_str() + (i * length), length));
+    EXPECT_EQ(0, memcmp(encoded[i].c_str(), in.c_str() + (i * length), length));
   }
 
   buffer::ptr enc[k + m];
@@ -705,7 +862,7 @@ TYPED_TEST(IsaErasureCodeTest, isa_xor_codec)
   EXPECT_EQ(5, cnt_cf);
 }
 
-TEST(IsaErasureCodeTest, create_ruleset)
+TEST_F(IsaErasureCodeTest, create_ruleset)
 {
   CrushWrapper *c = new CrushWrapper;
   c->create();
@@ -734,39 +891,9 @@ TEST(IsaErasureCodeTest, create_ruleset)
     }
   }
 
-  //
-  // The ruleid may be different from the ruleset when a crush rule is
-  // removed because the removed ruleid will be reused but the removed
-  // ruleset will not be reused. 
-  //
-  // This also asserts that the create_ruleset() method returns a
-  // ruleset and not a ruleid http://tracker.ceph.com/issues/9044
-  //
   {
     stringstream ss;
-    ErasureCodeIsaDefault isa;
-    map<std::string,std::string> parameters;
-    parameters["k"] = "2";
-    parameters["m"] = "2";
-    parameters["w"] = "8";
-    isa.init(parameters);
-    int FIRST = isa.create_ruleset("FIRST", *c, &ss);
-    int SECOND = isa.create_ruleset("SECOND", *c, &ss);
-    int FIRST_ruleid = c->get_rule_id("FIRST");
-    EXPECT_EQ(0, c->remove_rule(FIRST_ruleid));
-    int ruleset = isa.create_ruleset("myrule", *c, &ss);
-    EXPECT_NE(FIRST, ruleset);
-    EXPECT_NE(SECOND, ruleset);
-    EXPECT_NE(ruleset, c->get_rule_id("myrule"));
-    int SECOND_ruleid = c->get_rule_id("SECOND");
-    EXPECT_EQ(0, c->remove_rule(SECOND_ruleid));
-    int myrule_ruleid = c->get_rule_id("myrule");
-    EXPECT_EQ(0, c->remove_rule(myrule_ruleid));
-  }
-
-  {
-    stringstream ss;
-    ErasureCodeIsaDefault isa;
+    ErasureCodeIsaDefault isa(tcache);
     map<std::string,std::string> parameters;
     parameters["k"] = "2";
     parameters["m"] = "2";
@@ -790,7 +917,7 @@ TEST(IsaErasureCodeTest, create_ruleset)
   }
   {
     stringstream ss;
-    ErasureCodeIsaDefault isa;
+    ErasureCodeIsaDefault isa(tcache);
     map<std::string,std::string> parameters;
     parameters["k"] = "2";
     parameters["m"] = "2";
@@ -802,7 +929,7 @@ TEST(IsaErasureCodeTest, create_ruleset)
   }
   {
     stringstream ss;
-    ErasureCodeIsaDefault isa;
+    ErasureCodeIsaDefault isa(tcache);
     map<std::string,std::string> parameters;
     parameters["k"] = "2";
     parameters["m"] = "2";
@@ -828,9 +955,8 @@ int main(int argc, char **argv)
 
 /*
  * Local Variables:
- * compile-command: "cd ../.. ; make -j4 &&
- *   make unittest_erasure_code_isa &&
- *   valgrind --tool=memcheck --leak-check=full \
+ * compile-command: "cd ../.. ; make -j4 unittest_erasure_code_isa &&
+ *   libtool --mode=execute valgrind --tool=memcheck --leak-check=full \
  *      ./unittest_erasure_code_isa \
  *      --gtest_filter=*.* --log-to-stderr=true --debug-osd=20"
  * End:
